﻿using Hotline.Share.Dtos.Identity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Cache;
using XF.Domain.Dependency;

namespace Hotline.Identity
{
    public interface IIdentityDomainService
    {
        /// <summary>
        /// 校验用户是否真实
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        bool IsIdentityReal(LoginWithSignatureRequest request);

        /// <summary>
        /// 设置账户随机数
        /// </summary>
        /// <param name="username"></param>
        /// <param name="nonce"></param>
        void SetAccountNonce(string username, string nonce);
    }

    public class IdentityDomainService : IIdentityDomainService, IScopeDependency
    {
        private readonly ITypedCache<AccountNonce> _cacheAccountNonce;

        public IdentityDomainService(
            ITypedCache<AccountNonce> cacheAccountNonce)
        {
            _cacheAccountNonce = cacheAccountNonce;
        }

        /// <summary>
        /// 校验用户是否真实
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public bool IsIdentityReal(LoginWithSignatureRequest request)
        {
            /*
             *时间戳timestamp与服务器时间戳相差不能超过60s大于服务器时间戳
               随机数nonce60s内不能重复
             */
            if (string.IsNullOrEmpty(request.Nonce)) return false;
            var now = DateTimeOffset.UtcNow.ToUnixTimeMilliseconds();
            if (request.Timestamp >= now) return false;
            if ((now - request.Timestamp) >= 60000) return false;
            var nonce = _cacheAccountNonce.Get(request.Username)?.Nonce;
            if (!string.IsNullOrEmpty(nonce) && string.CompareOrdinal(nonce, request.Nonce) == 0) return false;
            return true;
        }

        /// <summary>
        /// 设置账户随机数
        /// </summary>
        /// <param name="username"></param>
        /// <param name="nonce"></param>
        public void SetAccountNonce(string username, string nonce)
        {
            _cacheAccountNonce.Set(username, new AccountNonce(nonce), TimeSpan.FromSeconds(60));
        }
    }
}
